import pytest
import responses

from datarobot.models import ComplianceDocTemplate


@pytest.fixture
def templates_list_response():
    return [
        {
            "id": "5bbdf361962d7435e2c447fc",
            "name": "template #1",
            "creatorId": "5a421957aadebb4f0df2f096",
            "creatorUsername": "admin@datarobot.com",
            "orgId": "5a421957aadebb4f0df2f091",
            "sections": [
                {
                    "contentId": u"HOW_TO_USE",
                    "sections": [],
                    "title": u"How To Use This Document",
                    "type": u"datarobot",
                },
                {"title": u"Table of Contents", "type": u"table_of_contents"},
                {
                    "title": u"Empty user section",
                    "type": u"user",
                    "regularText": u"",
                    "highlighted_text": u"",
                },
                {
                    "contentId": u"PREFACE",
                    "sections": [],
                    "title": u"DataRobot Model Development Documentation",
                    "type": u"datarobot",
                },
                {
                    "contentId": u"EXECUTIVE_SUMMARY_AND_MODEL_OVERVIEW",
                    "sections": [],
                    "title": u"Exec Summary And Model Overview",
                    "type": u"datarobot",
                },
            ],
        },
        {
            "id": "5bbdf361962d7435e2c447fd",
            "name": "template #2",
            "creatorId": "5a421957aadebb4f0df2f096",
            "creatorUsername": "admin@datarobot.com",
            "orgId": "5a421957aadebb4f0df2f091",
            "sections": [
                {
                    "contentId": u"HOW_TO_USE",
                    "sections": [],
                    "title": u"How To Use This Document",
                    "type": u"datarobot",
                },
                {"title": u"Table of Contents", "type": u"table_of_contents"},
                {
                    "contentId": u"PREFACE",
                    "sections": [],
                    "title": u"DataRobot Model Development Documentation",
                    "type": u"datarobot",
                },
                {
                    "contentId": u"EXECUTIVE_SUMMARY_AND_MODEL_OVERVIEW",
                    "sections": [],
                    "title": u"Exec Summary And Model Overview",
                    "type": u"datarobot",
                },
            ],
        },
    ]


@responses.activate
def test_list(templates_list_response):
    responses.add(
        responses.GET,
        "https://host_name.com/complianceDocTemplates/",
        json={
            "data": templates_list_response,
            "next": None,
            "previous": None,
            "count": len(templates_list_response),
        },
    )

    templates = ComplianceDocTemplate.list()

    for template, server_data in zip(templates, templates_list_response):
        assert template.id == server_data["id"]
        assert template.name == server_data["name"]
        assert template.creator_id == server_data["creatorId"]
        assert template.creator_username == server_data["creatorUsername"]
        assert template.org_id == server_data["orgId"]
        assert template.sections is not None


@responses.activate
def test_get(templates_list_response):
    server_data = templates_list_response[0]
    responses.add(
        responses.GET,
        "https://host_name.com/complianceDocTemplates/{}/".format(server_data["id"]),
        json=server_data,
    )
    template = ComplianceDocTemplate.get(server_data["id"])

    assert template.id == server_data["id"]
    assert template.name == server_data["name"]
    assert template.creator_id == server_data["creatorId"]
    assert template.creator_username == server_data["creatorUsername"]
    assert template.org_id == server_data["orgId"]
    assert template.sections is not None


@responses.activate
def test_get_no_org_id(templates_list_response):
    server_data = dict(templates_list_response[0], orgId=None)
    responses.add(
        responses.GET,
        "https://host_name.com/complianceDocTemplates/{}/".format(server_data["id"]),
        json=server_data,
    )
    template = ComplianceDocTemplate.get(server_data["id"])

    assert template.id == server_data["id"]
    assert template.name == server_data["name"]
    assert template.creator_id == server_data["creatorId"]
    assert template.creator_username == server_data["creatorUsername"]
    assert template.org_id is None
    assert template.sections is not None


@responses.activate
def test_update_name(templates_list_response):
    server_data = templates_list_response[0]
    responses.add(
        responses.GET,
        "https://host_name.com/complianceDocTemplates/{}/".format(server_data["id"]),
        json=server_data,
    )
    responses.add(
        responses.PATCH,
        "https://host_name.com/complianceDocTemplates/{}/".format(server_data["id"]),
        body="",
        status=204,
    )
    template = ComplianceDocTemplate.get(server_data["id"])
    template.update(name="updated name")

    assert template.name == "updated name"


@responses.activate
def test_update_section(templates_list_response):
    server_data = templates_list_response[0]
    responses.add(
        responses.GET,
        "https://host_name.com/complianceDocTemplates/{}/".format(server_data["id"]),
        json=server_data,
    )
    responses.add(
        responses.PATCH,
        "https://host_name.com/complianceDocTemplates/{}/".format(server_data["id"]),
        body="",
        status=204,
    )
    new_sections = [
        {"contentId": "HOW_TO_USE", "title": "How To Use", "type": "datarobot"},
        {"title": "Table of Contents", "type": "table_of_contents"},
        {"contentId": "PREFACE", "title": "Preface", "type": "datarobot"},
        {
            "contentId": "EXECUTIVE_SUMMARY_AND_MODEL_OVERVIEW",
            "title": "Summary",
            "type": "datarobot",
        },
        {
            "highlightedText": "Foo",
            "regularText": "Bar",
            "title": "Custom Section Added by User",
            "type": "user",
        },
    ]
    template = ComplianceDocTemplate.get(server_data["id"])
    template.update(sections=new_sections)

    assert template.sections[0]["contentId"] == "HOW_TO_USE"


@responses.activate
def test_create(templates_list_response):
    data = templates_list_response[0]
    responses.add(
        responses.POST,
        "https://host_name.com/complianceDocTemplates/",
        body="",
        status=201,
        adding_headers={
            "Location": "https://host_name.com/complianceDocTemplates/{}/".format(data["id"])
        },
    )
    responses.add(
        responses.GET,
        "https://host_name.com/complianceDocTemplates/{}/".format(data["id"]),
        json=data,
    )
    template = ComplianceDocTemplate.create(name=data["name"], sections=data["sections"])
    assert template.id == data["id"]
    assert template.name == data["name"]
    assert template.creator_id == data["creatorId"]
    assert template.creator_username == data["creatorUsername"]
    assert template.org_id == data["orgId"]
    assert template.sections is not None


@responses.activate
def test_get_default(templates_list_response):
    server_data = templates_list_response[0]
    responses.add(
        responses.GET, "https://host_name.com/complianceDocTemplates/default/", json=server_data
    )
    template = ComplianceDocTemplate.get_default()
    assert template.id is None
    assert template.creator_id is None
    assert template.creator_username is None
    assert template.org_id is None
    assert template.sections is not None
    assert template.name == "default"


@responses.activate
def test_sections_to_json_file(templates_list_response, tmpdir):
    server_data = templates_list_response[0]
    responses.add(
        responses.GET, "https://host_name.com/complianceDocTemplates/default/", json=server_data
    )
    responses.add(
        responses.POST,
        "https://host_name.com/complianceDocTemplates/",
        body="",
        status=201,
        headers={
            "Location": "https://host_name.com/complianceDocTemplates/{}/".format("new-template")
        },
    )
    responses.add(
        responses.GET,
        "https://host_name.com/complianceDocTemplates/new-template/",
        json=dict(server_data, name="new template"),
    )
    template = ComplianceDocTemplate.get_default()
    tmpfile = tmpdir.mkdir("sections").join("sections.json")
    template.sections_to_json_file(path=tmpfile.strpath)
    assert tmpfile.exists()

    new_template = ComplianceDocTemplate.create_from_json_file(
        name="new template", path=tmpfile.strpath
    )
    assert new_template.name == "new template"
    assert new_template.creator_id == server_data["creatorId"]
    assert new_template.creator_username == server_data["creatorUsername"]
    assert new_template.org_id == server_data["orgId"]
    assert new_template.sections is not None



import json

import responses

from datarobot.models.visualai import ImageActivationMap, ImageEmbedding


@responses.activate
def test_image_embeddings(
    visualai_embeddings,
    visualai_embeddings_url,
    project_server_data,
    project_url,
    project_id,
    model_id,
):
    responses.add(
        responses.GET,
        project_url,
        status=200,
        content_type="application/json",
        body=json.dumps(project_server_data),
    )
    responses.add(
        responses.GET,
        visualai_embeddings_url,
        status=200,
        content_type="application/json",
        body=json.dumps(visualai_embeddings),
    )
    img0, img1 = ImageEmbedding.list(project_id, model_id, "image")
    assert isinstance(img0, ImageEmbedding)
    assert img0.image.id == visualai_embeddings.get("embeddings")[0].get("imageId")
    assert img0.project_id == project_id
    assert img0.image.project_id == project_id
    assert img0.model_id == model_id
    assert img0.feature_name == "image"
    assert img0.position_x == visualai_embeddings.get("embeddings")[0].get("positionX")
    assert img0.position_y == visualai_embeddings.get("embeddings")[0].get("positionY")
    assert img0.actual_target_value == visualai_embeddings.get("embeddings")[0].get(
        "actualTargetValue"
    )
    assert isinstance(img1, ImageEmbedding)
    assert img1.image.id == visualai_embeddings.get("embeddings")[1].get("imageId")
    assert img1.project_id == project_id
    assert img1.image.project_id == project_id
    assert img1.model_id == model_id
    assert img1.feature_name == "image"
    assert img1.position_x == visualai_embeddings.get("embeddings")[1].get("positionX")
    assert img1.position_y == visualai_embeddings.get("embeddings")[1].get("positionY")
    assert img1.actual_target_value == visualai_embeddings.get("embeddings")[1].get(
        "actualTargetValue"
    )


@responses.activate
def test_image_activationmpas(
    visualai_activationmaps,
    visualai_activationmaps_url,
    project_server_data,
    project_url,
    project_id,
    model_id,
):
    responses.add(
        responses.GET,
        project_url,
        status=200,
        content_type="application/json",
        body=json.dumps(project_server_data),
    )
    responses.add(
        responses.GET,
        visualai_activationmaps_url,
        status=200,
        content_type="application/json",
        body=json.dumps(visualai_activationmaps),
    )
    img0, img1 = ImageActivationMap.list(project_id, model_id, "image")
    assert isinstance(img0, ImageActivationMap)
    assert img0.image.id == visualai_activationmaps.get("activationMaps")[0].get("imageId")
    assert img0.overlay_image.id == visualai_activationmaps.get("activationMaps")[0].get(
        "overlayImageId"
    )
    assert img0.project_id == project_id
    assert img0.image.project_id == project_id
    assert img0.overlay_image.project_id == project_id
    assert img0.model_id == model_id
    assert img0.feature_name == "image"
    assert img0.actual_target_value == visualai_activationmaps.get("activationMaps")[0].get(
        "actualTargetValue"
    )
    assert img0.predicted_target_value == visualai_activationmaps.get("activationMaps")[0].get(
        "predictedTargetValue"
    )
    assert img0.activation_values == visualai_activationmaps.get("activationMaps")[0].get(
        "activationValues"
    )
    assert isinstance(img1, ImageActivationMap)
    assert img1.image.id == visualai_activationmaps.get("activationMaps")[1].get("imageId")
    assert img1.overlay_image.id == visualai_activationmaps.get("activationMaps")[1].get("imageId")
    assert img1.project_id == project_id
    assert img1.image.project_id == project_id
    assert img1.overlay_image.project_id == project_id
    assert img1.model_id == model_id
    assert img1.feature_name == "image"
    assert img1.actual_target_value == visualai_activationmaps.get("activationMaps")[1].get(
        "actualTargetValue"
    )
    assert img1.predicted_target_value == visualai_activationmaps.get("activationMaps")[1].get(
        "predictedTargetValue"
    )
    assert img1.activation_values == visualai_activationmaps.get("activationMaps")[1].get(
        "activationValues"
    )



# -*- encoding: utf-8 -*-
from datetime import datetime
import json

from dateutil.tz import tzutc
import pandas as pd
import pytest
import responses

from datarobot import errors, Predictions
from datarobot.utils import parse_time
from tests.utils import add_response


@pytest.fixture
def sample_prediction():
    return {
        u"positiveClass": None,
        u"task": u"Regression",
        u"predictions": [
            {u"positiveProbability": None, u"prediction": 32.0, u"rowId": 0},
            {u"positiveProbability": None, u"prediction": 100.0, u"rowId": 1},
            {u"positiveProbability": None, u"prediction": 212.0, u"rowId": 2},
        ],
    }


@pytest.fixture
def predictions_regression_with_shap_explanations():
    return {
        "shapBaseValue": [31108.28125],
        "task": "Regression",
        "positiveClass": None,
        "predictions": [
            {
                "positiveProbability": None,
                "predictionExplanationMetadata": {"shapRemainingTotal": -6969.941378326501},
                "prediction": 27229.166666666668,
                "rowId": 0,
                "predictionExplanations": [
                    {
                        "featureValue": "2004",
                        "strength": 3090.8262156678165,
                        "feature": "YearMade",
                        "label": "SalePrice",
                    },
                    {
                        "featureValue": "999089",
                        "strength": -2788.4580943655205,
                        "feature": "MachineID",
                        "label": "SalePrice",
                    },
                    {
                        "featureValue": "2006",
                        "strength": -2286.181442643504,
                        "feature": "saledate (Year)",
                        "label": "SalePrice",
                    },
                ],
            },
            {
                "positiveProbability": None,
                "predictionExplanationMetadata": {"shapRemainingTotal": 13439.981135022792},
                "prediction": 60583.333333333336,
                "rowId": 1,
                "predictionExplanations": [
                    {
                        "featureValue": "117657",
                        "strength": 16035.069717943523,
                        "feature": "MachineID",
                        "label": "SalePrice",
                    },
                    {
                        "featureValue": "77",
                        "strength": 10105.427194858292,
                        "feature": "ModelID",
                        "label": "SalePrice",
                    },
                    {
                        "featureValue": "Medium",
                        "strength": 3432.899846833262,
                        "feature": "ProductSize",
                        "label": "SalePrice",
                    },
                ],
            },
        ],
    }


@pytest.fixture
def predictions_binary_with_shap_explanations():
    return {
        "shapBaseValue": [0.39640626311302185],
        "task": "Binary",
        "positiveClass": 1.0,
        "predictions": [
            {
                "positiveProbability": 0.311178247734139,
                "prediction": 0.0,
                "predictionExplanations": [
                    {
                        "featureValue": "9",
                        "strength": 0.05346482340175079,
                        "feature": "number_diagnoses",
                        "label": 1.0,
                    },
                    {
                        "featureValue": "0",
                        "strength": -0.049024358641319835,
                        "feature": "number_inpatient",
                        "label": 1.0,
                    },
                    {
                        "featureValue": "Surgery-Neuro",
                        "strength": -0.03165299590594699,
                        "feature": "medical_specialty",
                        "label": 1.0,
                    },
                ],
                "rowId": 0,
                "predictionValues": [
                    {"value": 0.3111782477000000, "label": 1.0},
                    {"value": 0.6888217522658611, "label": 0.0},
                ],
                # To test the case with empty metadata from API.
                "predictionExplanationMetadata": {"shapRemainingTotal": None},
                "predictionThreshold": 0.5,
            },
            {
                "positiveProbability": 0.2829861111111111,
                "prediction": 0.0,
                "predictionExplanations": [
                    {
                        "featureValue": "0",
                        "strength": -0.05744066866980976,
                        "feature": "number_inpatient",
                        "label": 1.0,
                    },
                    {
                        "featureValue": "6",
                        "strength": -0.04089034311313498,
                        "feature": "number_diagnoses",
                        "label": 1.0,
                    },
                    {
                        "featureValue": "[50-75)",
                        "strength": 0.03388593271674542,
                        "feature": "weight",
                        "label": 1.0,
                    },
                ],
                "rowId": 1,
                "predictionValues": [
                    {"value": 0.2829861111111111, "label": 1.0},
                    {"value": 0.7170138888888888, "label": 0.0},
                ],
                "predictionExplanationMetadata": {"shapRemainingTotal": -0.00887273281105585},
                "predictionThreshold": 0.5,
            },
        ],
    }


@pytest.fixture
def predictions_multilabel_json():
    predictions = {
        "positiveClass": None,
        "task": "Multilabel",
        "predictions": [
            {
                "predictionValues": [
                    {"threshold": 0.5, "value": 0.6, "label": "cat"},
                    {"threshold": 0.5, "value": 0.5, "label": u"🐕"},
                ],
                "prediction": ["cat"],
                "rowId": 0,
            },
            {
                "predictionValues": [
                    {"threshold": 0.5, "value": 0.1, "label": "cat"},
                    {"threshold": 0.5, "value": 0.4, "label": u"🐕"},
                ],
                "prediction": [],
                "rowId": 1,
            },
        ],
    }
    return predictions


@pytest.fixture
def predictions_with_shap_exp_csv():
    return """row_id,prediction,Explanation_1_feature_name,Explanation_1_feature_value,Explanation_1_strength,shap_remaining_total,shap_base_value
    0,27229.166666666668,YearMade,2004,3090.8262156678165,-6969.941378326501,31108.28125
    1,60583.333333333336,MachineID,117657,16035.069717943523,13439.981135022792,31108.28125
    2,11800.0,fiSecondaryDesc,nan,-14385.515413569377,-4922.765764714323,31108.28125
    3,17453.125,fiSecondaryDesc,nan,-7614.76113549681,-6040.3950427868795,31108.28125
    4,11539.473684210527,Hydraulics_Flow,Standard,-14743.391828540229,-4825.41571693096,31108.28125
    5,22555.555555555555,MachineID,1001274,-6044.702269899164,-2508.024220884521,31108.28125
    6,60583.333333333336,MachineID,772701,20612.841554550116,8862.20929841619,31108.28125
    7,31200.0,MachineID,902002,13999.795343335558,-13908.076521619252,31108.28125
    8,67863.63636363637,YearMade,2008,30190.432159295826,6564.919474920468,31108.28125
    9,21176.470588235294,ProductSize,Large,7127.206390025408,-17059.016865184112,31108.28125
    """


@pytest.fixture
def predictions_metadata_with_shap_exp(prediction_id, project_id, model_id, dataset_id):
    url = "http://localhost/api/v2/projects/{}/predictions/{}/".format(project_id, prediction_id)
    return {
        "url": url,
        "id": prediction_id,
        "projectId": project_id,
        "modelId": model_id,
        "predictionDatasetId": dataset_id,
        "actualValueColumn": None,
        "forecastPoint": None,
        "includesPredictionIntervals": False,
        "predictionIntervalsSize": None,
        "predictionsStartDate": None,
        "predictionsEndDate": None,
        "explanationAlgorithm": "shap",
        "maxExplanations": 1,
        "shapWarnings": {"mismatchRowCount": 1, "maxNormalizedMismatch": 0.01},
    }


@pytest.fixture
def sample_prediction_metadata(prediction_id):
    project_id = "59d92fe8962d7464ca8c6bd6"
    url = "http://localhost/api/v2/projects/{}/predictions/{}/".format(project_id, prediction_id)

    return {
        "projectId": project_id,
        "id": prediction_id,
        "predictionDatasetId": "a_dataset_id",
        "modelId": "a_model_id",
        "includesPredictionIntervals": False,
        "predictionIntervalsSize": None,
        "url": url,
        "forecastPoint": "2017-01-01T15:00:00Z",
        "predictionsStartDate": None,
        "predictionsEndDate": None,
    }


@pytest.fixture
def prediction_id():
    return "59d92fe8962d7464ca8c6bd6"


@pytest.fixture
def model_id():
    return "modelid"


@pytest.fixture
def dataset_id():
    return "datasetid"


@pytest.fixture
def predictions_url(project_id, prediction_id):
    return "https://host_name.com/projects/{}/predictions/{}/".format(project_id, prediction_id)


@pytest.fixture
def predictions_metadata_url(project_id):
    return "https://host_name.com/projects/{}/predictionsMetadata/".format(project_id)


@pytest.fixture
def predictions_metadata_by_id_url(predictions_metadata_url, prediction_id):
    return predictions_metadata_url + "{}/".format(prediction_id)


@responses.activate
def test_get__fetches_metadata(project_id, prediction_id, sample_prediction_metadata):
    metadata_url = "https://host_name.com/projects/{}/predictionsMetadata/{}/".format(
        project_id, prediction_id,
    )
    responses.add(
        responses.GET,
        metadata_url,
        body=json.dumps(sample_prediction_metadata),
        status=200,
        content_type="application/json",
    )

    preds = Predictions.get(project_id, prediction_id)

    assert preds.project_id == project_id
    assert preds.prediction_id == prediction_id
    assert preds.model_id == sample_prediction_metadata["modelId"]
    assert preds.dataset_id == sample_prediction_metadata["predictionDatasetId"]
    assert preds.includes_prediction_intervals == (
        sample_prediction_metadata["includesPredictionIntervals"]
    )
    assert preds.forecast_point == parse_time(sample_prediction_metadata["forecastPoint"])
    assert preds.predictions_start_date == parse_time(
        sample_prediction_metadata["predictionsStartDate"]
    )
    assert preds.predictions_end_date == parse_time(
        sample_prediction_metadata["predictionsEndDate"]
    )
    assert preds.explanation_algorithm is None
    assert preds.max_explanations is None
    assert preds.shap_warnings is None


@responses.activate
def test_get_predictions_with_shap_explanations(
    project_id, prediction_id, predictions_metadata_with_shap_exp, predictions_metadata_by_id_url,
):
    mocked_response = predictions_metadata_with_shap_exp
    add_response(url=predictions_metadata_by_id_url, body=mocked_response)

    predictions = Predictions.get(project_id, prediction_id)

    assert predictions.project_id == project_id
    assert predictions.prediction_id == prediction_id
    assert predictions.model_id == mocked_response["modelId"]
    assert predictions.dataset_id == mocked_response["predictionDatasetId"]
    assert predictions.actual_value_column is None
    assert predictions.forecast_point is None
    assert not predictions.includes_prediction_intervals
    assert predictions.predictions_start_date is None
    assert predictions.predictions_end_date is None
    assert predictions.explanation_algorithm == mocked_response["explanationAlgorithm"]
    assert predictions.max_explanations == mocked_response["maxExplanations"]
    assert predictions.shap_warnings is not None
    assert (
        predictions.shap_warnings["mismatch_row_count"]
        == mocked_response["shapWarnings"]["mismatchRowCount"]
    )
    assert (
        predictions.shap_warnings["max_normalized_mismatch"]
        == mocked_response["shapWarnings"]["maxNormalizedMismatch"]
    )


@responses.activate
def test_list__ok(project_id, model_id, prediction_id, dataset_id, predictions_metadata_url):
    stub = [
        {
            "url": "http://localhost/api/v2/projects/{}/predictions/{}/".format(
                project_id, prediction_id
            ),
            "id": prediction_id,
            "modelId": model_id,
            "predictionDatasetId": dataset_id,
            "includesPredictionIntervals": False,
            "forecastPoint": "2017-01-01T15:00:00Z",
        },
        {
            "url": "http://localhost/api/v2/projects/{}/predictions/{}/".format(
                project_id, prediction_id
            ),
            "id": prediction_id,
            "modelId": model_id,
            "predictionDatasetId": dataset_id,
            "includesPredictionIntervals": False,
            "predictionsStartDate": "2019-05-19T00:00:00.000000Z",
            "predictionsEndDate": "2019-05-20T00:00:00.000000Z",
            "actualValueColumn": "actuals",
        },
        {
            "url": "http://localhost/api/v2/projects/{}/predictions/{}/".format(
                project_id, prediction_id
            ),
            "id": prediction_id,
            "projectId": project_id,
            "modelId": model_id,
            "predictionDatasetId": dataset_id,
            "includesPredictionIntervals": False,
            "predictionsStartDate": None,
            "predictionsEndDate": None,
            "predictionIntervalsSize": None,
            "actualValueColumn": None,
            "forecastPoint": None,
            "explanationAlgorithm": "shap",
            "maxExplanations": 1,
            "shapWarnings": None,
        },
    ]
    add_response(url=predictions_metadata_url, body={"data": stub})

    predictions_list = Predictions.list(project_id)

    assert len(predictions_list) == 3
    prediction = predictions_list[0]
    assert prediction.project_id == project_id
    assert prediction.model_id == model_id
    assert prediction.dataset_id == dataset_id
    assert prediction.prediction_id == prediction_id
    assert not prediction.includes_prediction_intervals
    assert prediction.prediction_intervals_size is None
    assert prediction.forecast_point == datetime(2017, 1, 1, 15, tzinfo=tzutc())
    assert prediction.predictions_start_date is None
    assert prediction.predictions_end_date is None
    assert prediction.explanation_algorithm is None
    assert prediction.max_explanations is None
    assert prediction.shap_warnings is None
    bulk_pred_metadata = predictions_list[1]
    assert bulk_pred_metadata.forecast_point is None
    assert bulk_pred_metadata.predictions_start_date == datetime(2019, 5, 19, tzinfo=tzutc())
    assert bulk_pred_metadata.predictions_end_date == datetime(2019, 5, 20, tzinfo=tzutc())
    assert bulk_pred_metadata.actual_value_column == "actuals"
    assert bulk_pred_metadata.explanation_algorithm is None
    assert bulk_pred_metadata.max_explanations is None
    assert bulk_pred_metadata.shap_warnings is None
    predictions_with_shap_exp = predictions_list[2]
    assert predictions_with_shap_exp.project_id == project_id
    assert predictions_with_shap_exp.model_id == model_id
    assert predictions_with_shap_exp.dataset_id == dataset_id
    assert predictions_with_shap_exp.prediction_id == prediction_id
    assert not predictions_with_shap_exp.includes_prediction_intervals
    assert predictions_with_shap_exp.prediction_intervals_size is None
    assert predictions_with_shap_exp.forecast_point is None
    assert predictions_with_shap_exp.predictions_start_date is None
    assert predictions_with_shap_exp.predictions_end_date is None
    assert predictions_with_shap_exp.explanation_algorithm == "shap"
    assert predictions_with_shap_exp.max_explanations == 1
    assert predictions_with_shap_exp.shap_warnings is None


@responses.activate
def test_get_all_as_dataframe__ok(
    project_id,
    prediction_id,
    project_with_target_json,
    sample_prediction,
    sample_prediction_metadata,
    predictions_url,
    predictions_metadata_by_id_url,
):
    add_response(predictions_url, sample_prediction)
    add_response(predictions_metadata_by_id_url, sample_prediction_metadata)

    obj = Predictions.get(project_id, prediction_id)
    data_frame = obj.get_all_as_dataframe()

    assert isinstance(data_frame, pd.DataFrame)
    assert data_frame.shape == (len(sample_prediction["predictions"]), 2)
    assert list(data_frame.columns) == [
        "prediction",
        "row_id",
    ]


@responses.activate
def test_get_all_as_dataframe_regression_json_serializer(
    project_id,
    prediction_id,
    predictions_metadata_by_id_url,
    predictions_metadata_with_shap_exp,
    predictions_url,
    predictions_regression_with_shap_explanations,
):
    add_response(predictions_metadata_by_id_url, predictions_metadata_with_shap_exp)
    add_response(predictions_url, predictions_regression_with_shap_explanations)

    predictions = Predictions.get(project_id, prediction_id)
    data_frame = predictions.get_all_as_dataframe(serializer="json")
    assert isinstance(data_frame, pd.DataFrame)
    assert data_frame.shape == (2, 13)
    assert "row_id" in data_frame.columns
    assert "prediction" in data_frame.columns
    assert "shap_remaining_total" in data_frame.columns
    assert "shap_base_value" in data_frame.columns
    explanation_columns = [column for column in data_frame.columns if "Explanation_" in column]
    assert len(explanation_columns) == 9
    feature_name_columns = [column for column in explanation_columns if "feature_name" in column]
    assert len(feature_name_columns) == 3
    feature_value_columns = [column for column in explanation_columns if "feature_value" in column]
    assert len(feature_value_columns) == 3
    strength_columns = [column for column in explanation_columns if "strength" in column]
    assert len(strength_columns) == 3


@responses.activate
def test_get_all_as_dataframe_binary_json_serializer(
    project_id,
    prediction_id,
    predictions_metadata_by_id_url,
    predictions_metadata_with_shap_exp,
    predictions_url,
    predictions_binary_with_shap_explanations,
):
    add_response(predictions_metadata_by_id_url, predictions_metadata_with_shap_exp)
    add_response(predictions_url, predictions_binary_with_shap_explanations)

    predictions = Predictions.get(project_id, prediction_id)
    data_frame = predictions.get_all_as_dataframe(serializer="json")
    assert isinstance(data_frame, pd.DataFrame)
    assert data_frame.shape == (2, 18)
    assert "row_id" in data_frame.columns
    assert "prediction" in data_frame.columns
    assert "shap_remaining_total" in data_frame.columns
    assert "shap_base_value" in data_frame.columns
    assert "explained_class" in data_frame.columns
    explanation_columns = [column for column in data_frame.columns if "Explanation_" in column]
    assert len(explanation_columns) == 9
    feature_name_columns = [column for column in explanation_columns if "feature_name" in column]
    assert len(feature_name_columns) == 3
    feature_value_columns = [column for column in explanation_columns if "feature_value" in column]
    assert len(feature_value_columns) == 3
    strength_columns = [column for column in explanation_columns if "strength" in column]
    assert len(strength_columns) == 3


@responses.activate
def test_get_all_as_dataframe_multilabel_json_serializer(
    project_id,
    prediction_id,
    predictions_metadata_by_id_url,
    predictions_url,
    predictions_multilabel_json,
    sample_prediction_metadata,
):
    add_response(predictions_metadata_by_id_url, sample_prediction_metadata)
    add_response(predictions_url, predictions_multilabel_json)

    predictions = Predictions.get(project_id, prediction_id)
    actual = predictions.get_all_as_dataframe(serializer="json")

    expected = pd.DataFrame(
        [
            {
                u"row_id": 0,
                u"threshold_cat": 0.5,
                u"prediction_cat": 1,
                u"class_cat": 0.6,
                u"threshold_🐕": 0.5,
                u"prediction_🐕": 0,
                u"class_🐕": 0.5,
            },
            {
                u"row_id": 1,
                u"threshold_cat": 0.5,
                u"prediction_cat": 0,
                u"class_cat": 0.1,
                u"threshold_🐕": 0.5,
                u"prediction_🐕": 0,
                u"class_🐕": 0.4,
            },
        ]
    )

    pd.testing.assert_frame_equal(actual, expected[actual.columns])


@responses.activate
def test_get_all_as_dataframe_csv_serializer(
    project_id,
    prediction_id,
    predictions_url,
    predictions_metadata_by_id_url,
    predictions_metadata_with_shap_exp,
    predictions_with_shap_exp_csv,
):
    add_response(url=predictions_metadata_by_id_url, body=predictions_metadata_with_shap_exp)
    add_response(url=predictions_url, body=predictions_with_shap_exp_csv, content_type="text/csv")

    predictions = Predictions.get(project_id, prediction_id)
    data_frame = predictions.get_all_as_dataframe(serializer="csv")
    assert isinstance(data_frame, pd.DataFrame)
    assert data_frame.shape == (10, 7)
    assert "row_id" in data_frame.columns
    assert "prediction" in data_frame.columns
    assert "shap_remaining_total" in data_frame.columns
    assert "shap_base_value" in data_frame.columns
    explanation_columns = [column for column in data_frame.columns if "Explanation_1" in column]
    assert len(explanation_columns) == 3
    assert any(["feature_name" in column for column in explanation_columns])
    assert any(["feature_value" in column for column in explanation_columns])
    assert any(["strength" in column for column in explanation_columns])


@responses.activate
def test_get_not_found(project_id, prediction_id, predictions_metadata_by_id_url):
    add_response(predictions_metadata_by_id_url, "", status=404)
    with pytest.raises(errors.ClientError):
        Predictions.get(project_id, prediction_id)


@responses.activate
def test_get_as_data_frame_server_error(project_id, prediction_id, predictions_metadata_by_id_url):
    add_response(predictions_metadata_by_id_url, "", status=500)
    with pytest.raises(errors.ServerError):
        Predictions.get(project_id, prediction_id)


def test_repr():
    preds = Predictions("project_id", "prediction_id", "model_id", "dataset_id", False)

    assert (
        repr(preds) == "Predictions(prediction_id='prediction_id', project_id='project_id', "
        "model_id='model_id', dataset_id='dataset_id', "
        "includes_prediction_intervals=False, prediction_intervals_size=None, "
        "forecast_point=None, predictions_start_date=None, "
        "predictions_end_date=None, actual_value_column=None, "
        "explanation_algorithm=None, max_explanations=None, shap_warnings=None)"
    )



from copy import deepcopy
import json

import pytest
import responses

from datarobot import Deployment
from datarobot.enums import MODEL_REPLACEMENT_REASON
from datarobot.utils import from_api
from tests.utils import request_body_to_json


@responses.activate
@pytest.mark.usefixtures("deployment_get_response")
def test_model_replacement_validation(unittest_endpoint, deployment_data):
    deployment_id = deployment_data["id"]
    status = "passing"
    message = "Model can be used to replace the current model of the deployment."
    checks = {"targetName": {"status": "passing", "message": "Target name matches."}}

    url = "{}/deployments/{}/model/validation/".format(unittest_endpoint, deployment_id)
    responses.add(
        responses.POST,
        url,
        body=json.dumps({"status": status, "message": message, "checks": checks}),
        status=200,
        content_type="application/json",
    )

    deployment = Deployment.get(deployment_id)
    response = deployment.validate_replacement_model("5ca3936e0eb05d0155e2b7a5")

    request_body = request_body_to_json(responses.calls[1].request)
    assert request_body["modelId"] == "5ca3936e0eb05d0155e2b7a5"

    assert status == response[0]
    assert message == response[1]
    assert from_api(checks) == response[2]


@responses.activate
def test_model_replacement_submission_pred_int_disabled(unittest_endpoint, deployment_data):
    deployment_id = deployment_data["id"]
    new_model_id = "5ca3936e0eb05d0155e2b7a5"
    model_replacement_url = "{}/deployments/{}/model/".format(unittest_endpoint, deployment_id)
    deployment_url = "{}/deployments/{}/".format(unittest_endpoint, deployment_id)
    status_url = "{}/status_url".format(unittest_endpoint)
    settings_url = "{}/deployments/{}/settings/".format(unittest_endpoint, deployment_id)

    new_deployment_data = deepcopy(deployment_data)
    new_deployment_data["model"]["id"] = new_model_id
    responses.add(
        responses.GET,
        deployment_url,
        status=200,
        content_type="application/json",
        body=json.dumps(deployment_data),
    )
    responses.add(
        responses.PATCH, model_replacement_url, headers={"Location": status_url}, status=202
    )
    responses.add(responses.GET, status_url, headers={"Location": deployment_url}, status=303)
    responses.add(
        responses.GET,
        deployment_url,
        status=200,
        content_type="application/json",
        body=json.dumps(new_deployment_data),
    )
    responses.add(
        responses.GET,
        settings_url,
        status=200,
        content_type="application/json",
        body=json.dumps({"predictionIntervals": {"enabled": False, "percentiles": []}}),
    )

    deployment = Deployment.get(deployment_id)
    assert deployment.model["id"] == deployment_data["model"]["id"]
    deployment.replace_model(new_model_id, MODEL_REPLACEMENT_REASON.DATA_DRIFT)
    assert deployment.model["id"] == new_model_id


@responses.activate
def test_model_replacement_submission_pred_int_enabled(unittest_endpoint, deployment_data):
    deployment_id = deployment_data["id"]
    new_model_id = "5ca3936e0eb05d0155e2b7a5"
    model_replacement_url = "{}/deployments/{}/model/".format(unittest_endpoint, deployment_id)
    deployment_url = "{}/deployments/{}/".format(unittest_endpoint, deployment_id)
    status_url = "{}/status_url".format(unittest_endpoint)
    settings_url = "{}/deployments/{}/settings/".format(unittest_endpoint, deployment_id)

    new_deployment_data = deepcopy(deployment_data)
    new_deployment_data["model"]["id"] = new_model_id
    project_id = new_deployment_data["model"]["projectId"]
    responses.add(
        responses.GET,
        deployment_url,
        status=200,
        content_type="application/json",
        body=json.dumps(deployment_data),
    )
    responses.add(
        responses.PATCH, model_replacement_url, headers={"Location": status_url}, status=202
    )
    responses.add(responses.GET, status_url, headers={"Location": deployment_url}, status=303)
    responses.add(
        responses.GET,
        deployment_url,
        status=200,
        content_type="application/json",
        body=json.dumps(new_deployment_data),
    )
    responses.add(
        responses.GET,
        settings_url,
        status=200,
        content_type="application/json",
        body=json.dumps({"predictionIntervals": {"enabled": True, "percentiles": [80]}}),
    )
    responses.add(
        responses.POST,
        "{}/projects/{}/models/{}/predictionIntervals/".format(
            unittest_endpoint, project_id, new_model_id
        ),
        status=202,
        headers={"Location": "{}/projects/{}/jobs/1/".format(unittest_endpoint, project_id)},
    )
    responses.add(
        responses.GET,
        "{}/projects/{}/jobs/1/".format(unittest_endpoint, project_id),
        status=303,
        content_type="application/json",
        body=json.dumps(
            {
                "status": u"COMPLETED",
                "model_id": new_model_id,
                "is_blocked": False,
                "url": "{}/projects/{}/jobs/1/".format(unittest_endpoint, project_id),
                "job_type": "calculate_prediction_intervals",
                "project_id": project_id,
                "id": 1,
            }
        ),
        headers={"Location": "{}/projects/{}/jobs/1/result/".format(unittest_endpoint, project_id)},
    )
    responses.add(responses.PATCH, settings_url, headers={"Location": status_url}, status=202)
    responses.add(responses.GET, status_url, headers={"Location": deployment_url}, status=303)

    deployment = Deployment.get(deployment_id)
    assert deployment.model["id"] == deployment_data["model"]["id"]
    deployment.replace_model(new_model_id, MODEL_REPLACEMENT_REASON.DATA_DRIFT)
    assert deployment.model["id"] == new_model_id



# coding: utf-8
import json

import pytest
import responses
from trafaret import DataError

from datarobot import CalendarFile, enums, Project, SharingAccess
from datarobot.errors import ClientError, IllegalFileName
from tests.test_helpers import fixture_file_path

_base_url = "https://host_name.com/calendars/"
_valid_calendar_id = "calId"
_invalid_calendar_id = "badCalId"
_valid_calendar_url = "{0}{1}/".format(_base_url, _valid_calendar_id)
_invalid_calendar_url = "{0}{1}/".format(_base_url, _invalid_calendar_id)

calendar_dict = {
    "numEvents": 5,
    "name": "calendar.csv",
    "calendarStartDate": "2018-01-01",
    "created": "2018-07-16T17:28:13.929687Z",
    "numEventTypes": 5,
    "source": "calendar.csv",
    "calendarEndDate": "2018-07-04",
    "projectIds": [],
    "Id": "calId",
    "role": "OWNER",
    "multiseriesIdColumns": None,
}

calendar_dict_with_name = {
    "numEvents": 5,
    "name": "My Calendar",
    "calendarStartDate": "2018-01-01",
    "created": "2018-07-16T17:28:13.929687Z",
    "numEventTypes": 5,
    "source": "calendar.csv",
    "calendarEndDate": "2018-07-04",
    "projectIds": [],
    "Id": "calId",
    "role": "OWNER",
    "multiseriesIdColumns": None,
}


multiseries_calendar_dict = {
    "numEvents": 32,
    "name": "multiseries_calendar.csv",
    "calendarStartDate": "2019-01-01",
    "created": "2019-10-17T17:28:13.929687Z",
    "numEventTypes": 5,
    "source": "multiseries_calendar.csv",
    "calendarEndDate": "2019-12-26",
    "projectIds": [],
    "Id": "calId",
    "role": "OWNER",
    "multiseriesIdColumns": ["series"],
}


multiseries_calendar_dict_with_name = {
    "numEvents": 32,
    "name": "My Multiseries Calendar",
    "calendarStartDate": "2019-01-01",
    "created": "2019-10-17T17:28:13.929687Z",
    "numEventTypes": 5,
    "source": "multiseries_calendar.csv",
    "calendarEndDate": "2019-12-26",
    "projectIds": [],
    "Id": "calId",
    "role": "OWNER",
    "multiseriesIdColumns": ["series"],
}

allowed_country_codes = [
    {"code": "UK", "name": "United Kingdom"},
    {"code": "US", "name": "United States"},
]


@pytest.fixture
def calendar_json():
    return json.dumps(calendar_dict)


@pytest.fixture
def calendar_with_name_json():
    return json.dumps(calendar_dict_with_name)


@pytest.fixture
def multiseries_calendar_json():
    return json.dumps(multiseries_calendar_dict)


@pytest.fixture
def multiseries_calendar_with_name_json():
    return json.dumps(multiseries_calendar_dict_with_name)


@pytest.fixture
def calendar_list_json():
    cal_list = []
    for i in range(1, 11):
        cal_copy = calendar_dict.copy()
        cal_copy["name"] = "cal_name{0}".format(i)
        cal_list.append(cal_copy)
    return json.dumps({"count": len(cal_list), "data": cal_list, "next": None, "previous": None})


@pytest.fixture
def multiseries_calendar_list_json():
    cal_list = []
    for i in range(1, 11):
        cal_copy = multiseries_calendar_dict.copy()
        cal_copy["name"] = "cal_name{0}".format(i)
        cal_list.append(cal_copy)
    return json.dumps({"count": len(cal_list), "data": cal_list, "next": None, "previous": None})


@pytest.fixture
def allowed_countries_list_json():
    return json.dumps(
        {
            "count": len(allowed_country_codes),
            "previous": None,
            "data": allowed_country_codes,
            "next": None,
        }
    )


@pytest.fixture
def calendar_access_control_list_json():
    ac_list = [
        {
            "userId": "ac{0}".format(i),
            "username": "user{0}".format(i),
            "canShare": True,
            "role": "OWNER",
        }
        for i in range(1, 11)
    ]
    return json.dumps({"count": len(ac_list), "data": ac_list, "next": None, "previous": None})


@pytest.fixture
def calendar_invalid_json():
    return json.dumps({"message": "Not Found"})


@responses.activate
def test_calendar_create(calendar_json):
    responses.add(
        responses.POST,
        _base_url + "fileUpload/",
        body="",
        status=202,
        adding_headers={"Location": "https://host_name.com/status/some-status/"},
        content_type="application_json",
    )
    responses.add(
        responses.GET,
        "https://host_name.com/status/some-status/",
        body="",
        status=303,
        adding_headers={"Location": _valid_calendar_url},
        content_type="application/json",
    )
    responses.add(
        responses.GET,
        _valid_calendar_url,
        body=calendar_json,
        status=200,
        content_type="application/json",
    )
    calendar = CalendarFile.create(fixture_file_path("calendar.csv"))
    assert isinstance(calendar, CalendarFile)
    assert calendar.id == "calId"
    assert calendar.num_events == 5
    assert calendar.name == "calendar.csv"
    assert calendar.calendar_start_date == "2018-01-01"
    assert calendar.calendar_end_date == "2018-07-04"
    assert calendar.created == "2018-07-16T17:28:13.929687Z"
    assert calendar.num_event_types == 5
    assert calendar.project_ids == []
    assert calendar.role == "OWNER"
    assert not calendar.multiseries_id_columns


def _test_multiseries_calendar_create(multiseries_calendar_json, name=None):
    responses.add(
        responses.POST,
        _base_url + "fileUpload/",
        body="",
        status=202,
        adding_headers={"Location": "https://host_name.com/status/some-status/"},
        content_type="application_json",
    )
    responses.add(
        responses.GET,
        "https://host_name.com/status/some-status/",
        body="",
        status=303,
        adding_headers={"Location": _valid_calendar_url},
        content_type="application/json",
    )
    responses.add(
        responses.GET,
        _valid_calendar_url,
        body=multiseries_calendar_json,
        status=200,
        content_type="application/json",
    )
    calendar = CalendarFile.create(
        fixture_file_path("multiseries_calendar.csv"), calendar_name=name
    )
    assert isinstance(calendar, CalendarFile)
    assert calendar.id == "calId"
    assert calendar.num_events == 32
    assert calendar.name == name if name else "multiseries_calendar.csv"
    assert calendar.calendar_start_date == "2019-01-01"
    assert calendar.calendar_end_date == "2019-12-26"
    assert calendar.created == "2019-10-17T17:28:13.929687Z"
    assert calendar.num_event_types == 5
    assert calendar.project_ids == []
    assert calendar.role == "OWNER"
    assert calendar.multiseries_id_columns == ["series"]


@responses.activate
def test_multiseries_calendar_create(multiseries_calendar_json):
    _test_multiseries_calendar_create(multiseries_calendar_json)


@responses.activate
def test_multiseries_calendar_create_with_name(multiseries_calendar_with_name_json):
    _test_multiseries_calendar_create(
        multiseries_calendar_with_name_json, name="My Multiseries Calendar"
    )


def test_calendar_create_non_existent_file():
    with pytest.raises(ValueError):
        CalendarFile.create("bad_file_path")


def test_calendar_create_not_a_string():
    with pytest.raises(ValueError):
        CalendarFile.create({"this definitely": 'isn"t a string'})


def test_calendar_create_multiseries_id_not_a_list():
    with pytest.raises(ValueError) as excinfo:
        CalendarFile.create(
            fixture_file_path("multiseries_calendar.csv"), multiseries_id_columns="bad_id"
        )
    assert str(excinfo.value) == "Expected list of str for multiseries_id_columns, got: bad_id"


@pytest.fixture
def project_without_partition(project_without_target_data):
    project_without_target_data["partition"] = None
    return Project(**project_without_target_data)


@responses.activate
def test_calendar_create_with_name(calendar_with_name_json):
    responses.add(
        responses.POST,
        _base_url + "fileUpload/",
        body="",
        status=202,
        adding_headers={"Location": "https://host_name.com/status/some-status/"},
        content_type="application_json",
    )
    responses.add(
        responses.GET,
        "https://host_name.com/status/some-status/",
        body="",
        status=303,
        adding_headers={"Location": _valid_calendar_url},
        content_type="application/json",
    )
    responses.add(
        responses.GET,
        _valid_calendar_url,
        body=calendar_with_name_json,
        status=200,
        content_type="application/json",
    )
    calendar = CalendarFile.create(fixture_file_path("calendar.csv"), calendar_name="My Calendar")
    assert isinstance(calendar, CalendarFile)
    assert calendar.id == "calId"
    assert calendar.num_events == 5
    assert calendar.name == "My Calendar"
    assert calendar.calendar_start_date == "2018-01-01"
    assert calendar.calendar_end_date == "2018-07-04"
    assert calendar.created == "2018-07-16T17:28:13.929687Z"
    assert calendar.num_event_types == 5
    assert calendar.project_ids == []
    assert calendar.role == "OWNER"


@responses.activate
def test_calendar_get(calendar_json):
    responses.add(
        responses.GET,
        _valid_calendar_url,
        body=calendar_json,
        status=200,
        content_type="application/json",
    )
    calendar = CalendarFile.get(_valid_calendar_id)
    assert isinstance(calendar, CalendarFile)
    assert calendar.id == "calId"
    assert calendar.num_events == 5
    assert calendar.name == "calendar.csv"
    assert calendar.calendar_start_date == "2018-01-01"
    assert calendar.calendar_end_date == "2018-07-04"
    assert calendar.created == "2018-07-16T17:28:13.929687Z"
    assert calendar.num_event_types == 5
    assert calendar.project_ids == []
    assert calendar.role == "OWNER"


@responses.activate
def test_create_calendar_from_country_code(calendar_json):
    calendar_params = {
        "countryCode": "US",
        "startDate": "2018-01-01",
        "endDate": "2018-07-04",
    }
    responses.add(
        responses.POST,
        _base_url + "fromCountryCode/",
        body=json.dumps(calendar_params),
        status=202,
        content_type="application/json",
        adding_headers={"Location": "https://host_name.com/status/some-status/"},
    )
    responses.add(
        responses.GET,
        "https://host_name.com/status/some-status/",
        body="",
        status=303,
        adding_headers={"Location": _valid_calendar_url},
        content_type="application/json",
    )
    responses.add(
        responses.GET,
        _valid_calendar_url,
        body=calendar_json,
        status=200,
        content_type="application/json",
    )
    generated_calendar = CalendarFile.create_calendar_from_country_code(
        calendar_params["countryCode"], calendar_params["startDate"], calendar_params["endDate"]
    )
    assert isinstance(generated_calendar, CalendarFile)
    assert generated_calendar.id == "calId"
    assert generated_calendar.role == "OWNER"
    assert generated_calendar.num_events == 5
    assert generated_calendar.name == "calendar.csv"
    assert generated_calendar.calendar_start_date == "2018-01-01"
    assert generated_calendar.calendar_end_date == "2018-07-04"
    assert generated_calendar.created == "2018-07-16T17:28:13.929687Z"
    assert generated_calendar.num_event_types == 5
    assert generated_calendar.project_ids == []
    assert not generated_calendar.multiseries_id_columns


@responses.activate
def test_allowed_countries_list(allowed_countries_list_json):
    responses.add(
        responses.GET,
        "https://host_name.com/calendarCountryCodes/",
        body=allowed_countries_list_json,
    )
    allowed_countries_list = CalendarFile.get_allowed_country_codes()
    assert sorted(allowed_countries_list, key=lambda x: x["name"]) == allowed_country_codes


@responses.activate
def test_calendar_invalid_get(calendar_invalid_json):
    responses.add(responses.GET, _invalid_calendar_url, body=calendar_invalid_json)
    with pytest.raises(DataError):
        CalendarFile.get(_invalid_calendar_id)


@responses.activate
def test_calendar_list(calendar_list_json):
    responses.add(responses.GET, _base_url, body=calendar_list_json)
    cal_list = CalendarFile.list()
    assert len(cal_list) == 10
    for item in cal_list:
        assert isinstance(item, CalendarFile)


@responses.activate
def test_multiseries_calendar_list(multiseries_calendar_list_json):
    responses.add(responses.GET, _base_url, body=multiseries_calendar_list_json)
    cal_list = CalendarFile.list()
    assert len(cal_list) == 10
    for item in cal_list:
        assert isinstance(item, CalendarFile)


@responses.activate
def test_calendar_delete_valid():
    responses.add(responses.DELETE, _valid_calendar_url, status=204)
    assert CalendarFile.delete("calId") is None


@responses.activate
def test_calendar_delete_invalid():
    responses.add(responses.DELETE, _invalid_calendar_url, status=404)
    with pytest.raises(ClientError):
        CalendarFile.delete(_invalid_calendar_id)


@responses.activate
def test_calendar_update_name_valid():
    responses.add(responses.PATCH, _valid_calendar_url, status=200)
    assert CalendarFile.update_name(_valid_calendar_id, "new name") == 200


@responses.activate
def test_calendar_update_name_invalid_calendar():
    responses.add(responses.PATCH, _invalid_calendar_url, status=404)
    with pytest.raises(ClientError):
        CalendarFile.update_name(_invalid_calendar_id, "new name")


@responses.activate
def test_calendar_update_name_invalid_name():
    with pytest.raises(IllegalFileName):
        CalendarFile.update_name(_valid_calendar_id, "£ ¥\n§ ¾")


@responses.activate
def test_calendar_sharing_valid():
    responses.add(responses.PATCH, _valid_calendar_url + "accessControl/", status=200)
    payload = [SharingAccess("valid_username@datarobot.com", enums.SHARING_ROLE.READ_WRITE)]
    assert CalendarFile.share(_valid_calendar_id, payload) == 200


@responses.activate
def test_calendar_sharing_invalid_username():
    responses.add(responses.PATCH, _invalid_calendar_url + "accessControl/", status=422)
    payload = [SharingAccess("invalid_valid_username@datarobot.com", enums.SHARING_ROLE.READ_WRITE)]
    with pytest.raises(ClientError):
        CalendarFile.share(_invalid_calendar_id, payload)


def test_calendar_sharing_invalid_access_list():
    # invalid access_list
    payload = {"look at me, I'm a": "dict"}
    with pytest.raises(AssertionError):
        CalendarFile.share(_valid_calendar_id, payload)
    # invalid items in access_list
    payload = [i for i in range(10)]
    with pytest.raises(AssertionError):
        CalendarFile.share(_valid_calendar_id, payload)


@responses.activate
def test_calendar_sharing_access_list(calendar_access_control_list_json):
    responses.add(
        responses.GET,
        _valid_calendar_url + "accessControl/",
        body=calendar_access_control_list_json,
    )
    access_control_list = CalendarFile.get_access_list(_valid_calendar_id)
    assert len(access_control_list) == 10
    for item in access_control_list:
        assert isinstance(item, SharingAccess)


@responses.activate
def test_calendar_sharing_access_list_user_no_access():
    responses.add(responses.GET, _valid_calendar_url + "accessControl/", status=404)
    with pytest.raises(ClientError):
        CalendarFile.get_access_list(_valid_calendar_id)


def test_retrieve_calendar_id_project_pre_aim(project_without_partition):
    assert project_without_partition.calendar_id is None



import responses

from datarobot.errors import InvalidUsageError
from tests.utils import request_body_to_json


@responses.activate
def test_set_target(
    datetime_partition_spec,
    holdout_start_date,
    project_with_target_json,
    project_without_target,
    unittest_endpoint,
    project_id,
):
    project_url = "{}/projects/{}/".format(unittest_endpoint, project_id)
    aim_url = project_url + "aim/"
    status_url = "{}/status/some-status/".format(unittest_endpoint)

    responses.add(
        responses.PATCH,
        aim_url,
        body="",
        status=202,
        adding_headers={"Location": status_url},
        content_type="application/json",
    )

    responses.add(
        responses.GET,
        status_url,
        body="",
        status=303,
        adding_headers={"Location": project_url},
        content_type="application/json",
    )
    responses.add(
        responses.GET,
        project_url,
        body=project_with_target_json,
        status=200,
        content_type="application/json",
    )

    expected_backtests = [
        {
            "index": bt.index,
            "gapDuration": bt.gap_duration,
            "validationStartDate": bt.validation_start_date.isoformat(),
            "validationDuration": bt.validation_duration,
        }
        for bt in datetime_partition_spec.backtests
    ]
    expected_payload = {
        "datetimePartitionColumn": datetime_partition_spec.datetime_partition_column,
        "useTimeSeries": datetime_partition_spec.use_time_series,
        "defaultToKnownInAdvance": datetime_partition_spec.default_to_known_in_advance,
        "defaultToDoNotDerive": datetime_partition_spec.default_to_do_not_derive,
        "autopilotDataSelectionMethod": datetime_partition_spec.autopilot_data_selection_method,
        "validationDuration": datetime_partition_spec.validation_duration,
        "holdoutStartDate": holdout_start_date.isoformat(),
        "holdoutDuration": datetime_partition_spec.holdout_duration,
        "gapDuration": datetime_partition_spec.gap_duration,
        "numberOfBacktests": datetime_partition_spec.number_of_backtests,
        "backtests": expected_backtests,
        "cvMethod": "datetime",
        "target": "target",
        "mode": "auto",
        "quickrun": True,
        "modelSplits": 2,
        "allowPartialHistoryTimeSeriesPredictions": (
            datetime_partition_spec.allow_partial_history_time_series_predictions
        ),
    }
    project_without_target.set_target("target", partitioning_method=datetime_partition_spec)
    assert responses.calls[0].request.method == "PATCH"
    actual_request = request_body_to_json(responses.calls[0].request)

    assert set(actual_request.keys()) == set(expected_payload.keys())
    assert actual_request == expected_payload


@responses.activate
def test_time_series_set_target(
    time_series_partition_spec,
    holdout_start_date,
    project_with_target_json,
    project_without_target,
    unittest_endpoint,
    project_id,
):
    project_url = "{}/projects/{}/".format(unittest_endpoint, project_id)
    aim_url = project_url + "aim/"
    status_url = "{}/status/some-status/".format(unittest_endpoint)

    time_series_partition_spec.calendar_id = project_id

    responses.add(
        responses.PATCH,
        aim_url,
        body="",
        status=202,
        adding_headers={"Location": status_url},
        content_type="application/json",
    )

    responses.add(
        responses.GET,
        status_url,
        body="",
        status=303,
        adding_headers={"Location": project_url},
        content_type="application/json",
    )
    responses.add(
        responses.GET,
        project_url,
        body=project_with_target_json,
        status=200,
        content_type="application/json",
    )

    expected_backtests = [
        {
            "index": bt.index,
            "gapDuration": bt.gap_duration,
            "validationStartDate": bt.validation_start_date.isoformat(),
            "validationDuration": bt.validation_duration,
        }
        for bt in time_series_partition_spec.backtests
    ]
    expected_payload = {
        "datetimePartitionColumn": time_series_partition_spec.datetime_partition_column,
        "useTimeSeries": time_series_partition_spec.use_time_series,
        "defaultToKnownInAdvance": time_series_partition_spec.default_to_known_in_advance,
        "defaultToDoNotDerive": time_series_partition_spec.default_to_do_not_derive,
        "featureDerivationWindowStart": time_series_partition_spec.feature_derivation_window_start,
        "featureDerivationWindowEnd": time_series_partition_spec.feature_derivation_window_end,
        "forecastWindowStart": time_series_partition_spec.forecast_window_start,
        "forecastWindowEnd": time_series_partition_spec.forecast_window_end,
        "autopilotDataSelectionMethod": time_series_partition_spec.autopilot_data_selection_method,
        "validationDuration": time_series_partition_spec.validation_duration,
        "holdoutStartDate": holdout_start_date.isoformat(),
        "holdoutDuration": time_series_partition_spec.holdout_duration,
        "gapDuration": time_series_partition_spec.gap_duration,
        "numberOfBacktests": time_series_partition_spec.number_of_backtests,
        "backtests": expected_backtests,
        "cvMethod": "datetime",
        "target": "target",
        "mode": "auto",
        "quickrun": True,
        "calendarId": time_series_partition_spec.calendar_id,
        "allowPartialHistoryTimeSeriesPredictions": (
            time_series_partition_spec.allow_partial_history_time_series_predictions
        ),
    }
    project_without_target.set_target("target", partitioning_method=time_series_partition_spec)

    assert responses.calls[0].request.method == "PATCH"
    actual_request = request_body_to_json(responses.calls[0].request)
    assert set(actual_request.keys()) == set(expected_payload.keys())
    assert actual_request == expected_payload


@responses.activate
def test_irregular_time_series_set_target(
    irregular_time_series_partition_spec,
    holdout_start_date,
    project_without_target,
    project_with_target_json,
    unittest_endpoint,
    project_id,
):
    project_url = "{}/projects/{}/".format(unittest_endpoint, project_id)
    aim_url = project_url + "aim/"
    status_url = "{}/status/some-status/".format(unittest_endpoint)
    spec = irregular_time_series_partition_spec

    responses.add(
        responses.PATCH,
        aim_url,
        body="",
        status=202,
        adding_headers={"Location": status_url},
        content_type="application/json",
    )
    responses.add(
        responses.GET,
        status_url,
        body="",
        status=303,
        adding_headers={"Location": project_url},
        content_type="application/json",
    )
    responses.add(
        responses.GET,
        project_url,
        body=project_with_target_json,
        status=200,
        content_type="application/json",
    )
    expected_backtests = [
        {
            "index": bt.index,
            "gapDuration": bt.gap_duration,
            "validationStartDate": bt.validation_start_date.isoformat(),
            "validationDuration": bt.validation_duration,
        }
        for bt in irregular_time_series_partition_spec.backtests
    ]
    expected_payload = {
        "datetimePartitionColumn": spec.datetime_partition_column,
        "useTimeSeries": spec.use_time_series,
        "defaultToKnownInAdvance": spec.default_to_known_in_advance,
        "defaultToDoNotDerive": spec.default_to_do_not_derive,
        "featureDerivationWindowStart": spec.feature_derivation_window_start,
        "featureDerivationWindowEnd": spec.feature_derivation_window_end,
        "forecastWindowStart": spec.forecast_window_start,
        "forecastWindowEnd": spec.forecast_window_end,
        "autopilotDataSelectionMethod": spec.autopilot_data_selection_method,
        "holdoutStartDate": holdout_start_date.isoformat(),
        "holdoutDuration": spec.holdout_duration,
        "gapDuration": spec.gap_duration,
        "numberOfBacktests": spec.number_of_backtests,
        "backtests": expected_backtests,
        "cvMethod": "datetime",
        "target": "target",
        "mode": "auto",
        "quickrun": True,
        "allowPartialHistoryTimeSeriesPredictions": (
            spec.allow_partial_history_time_series_predictions
        ),
    }
    project_without_target.set_target("target", partitioning_method=spec)

    assert responses.calls[0].request.method == "PATCH"
    actual_request = request_body_to_json(responses.calls[0].request)
    assert set(actual_request.keys()) == set(expected_payload.keys())
    assert actual_request == expected_payload


@responses.activate
def test_multiseries_set_target(
    time_series_partition_spec,
    holdout_start_date,
    project_with_target_json,
    project_without_target,
    unittest_endpoint,
    project_id,
):
    time_series_partition_spec.multiseries_id_columns = ["series_id"]
    project_url = "{}/projects/{}/".format(unittest_endpoint, project_id)
    aim_url = project_url + "aim/"
    status_url = "{}/status/some-status/".format(unittest_endpoint)
    multi_prop_url = "{}features/{}/multiseriesProperties/".format(
        project_url, time_series_partition_spec.datetime_partition_column
    )
    multi_prop_json = {
        "datetimePartitionColumn": time_series_partition_spec.datetime_partition_column,
        "detectedMultiseriesIdColumns": [
            {"multiseriesIdColumns": ["series_id"], "timeStep": 1, "timeUnit": "DAY"}
        ],
    }
    responses.add(responses.GET, multi_prop_url, json=multi_prop_json)
    date_feat_json = {
        "projectId": project_id,
        "name": time_series_partition_spec.datetime_partition_column,
        "id": 2,
        "timeSeriesEligible": False,
        "featureType": "date",
        "timeSeriesEligibilityReason": "notUnique",
        "lowInformation": False,
        "uniqueCount": 500,
    }
    responses.add(
        responses.GET,
        "{}features/{}/".format(project_url, time_series_partition_spec.datetime_partition_column),
        json=date_feat_json,
    )

    responses.add(
        responses.PATCH,
        aim_url,
        body="",
        status=202,
        adding_headers={"Location": status_url},
        content_type="application/json",
    )

    responses.add(
        responses.GET,
        status_url,
        body="",
        status=303,
        adding_headers={"Location": project_url},
        content_type="application/json",
    )
    responses.add(
        responses.GET,
        project_url,
        body=project_with_target_json,
        status=200,
        content_type="application/json",
    )

    expected_backtests = [
        {
            "index": bt.index,
            "gapDuration": bt.gap_duration,
            "validationStartDate": bt.validation_start_date.isoformat(),
            "validationDuration": bt.validation_duration,
        }
        for bt in time_series_partition_spec.backtests
    ]
    expected_payload = {
        "datetimePartitionColumn": time_series_partition_spec.datetime_partition_column,
        "multiseriesIdColumns": time_series_partition_spec.multiseries_id_columns,
        "useTimeSeries": time_series_partition_spec.use_time_series,
        "defaultToKnownInAdvance": time_series_partition_spec.default_to_known_in_advance,
        "defaultToDoNotDerive": time_series_partition_spec.default_to_do_not_derive,
        "featureDerivationWindowStart": time_series_partition_spec.feature_derivation_window_start,
        "featureDerivationWindowEnd": time_series_partition_spec.feature_derivation_window_end,
        "forecastWindowStart": time_series_partition_spec.forecast_window_start,
        "forecastWindowEnd": time_series_partition_spec.forecast_window_end,
        "autopilotDataSelectionMethod": time_series_partition_spec.autopilot_data_selection_method,
        "validationDuration": time_series_partition_spec.validation_duration,
        "holdoutStartDate": holdout_start_date.isoformat(),
        "holdoutDuration": time_series_partition_spec.holdout_duration,
        "gapDuration": time_series_partition_spec.gap_duration,
        "numberOfBacktests": time_series_partition_spec.number_of_backtests,
        "backtests": expected_backtests,
        "cvMethod": "datetime",
        "target": "target",
        "mode": "auto",
        "quickrun": True,
        "allowPartialHistoryTimeSeriesPredictions": (
            time_series_partition_spec.allow_partial_history_time_series_predictions
        ),
    }
    project_without_target.set_target("target", partitioning_method=time_series_partition_spec)

    assert responses.calls[2].request.method == "PATCH"
    actual_request = request_body_to_json(responses.calls[2].request)
    assert set(actual_request.keys()) == set(expected_payload.keys())
    assert actual_request == expected_payload


@responses.activate
def test_use_cross_series_features(
    time_series_partition_spec,
    holdout_start_date,
    project_with_target_json,
    project_without_target,
    unittest_endpoint,
    project_id,
):
    time_series_partition_spec.multiseries_id_columns = ["series_id"]
    time_series_partition_spec.use_cross_series_features = True
    time_series_partition_spec.aggregation_type = "total"

    project_url = "{}/projects/{}/".format(unittest_endpoint, project_id)
    aim_url = project_url + "aim/"
    status_url = "{}/status/some-status/".format(unittest_endpoint)
    multi_prop_url = "{}features/{}/multiseriesProperties/".format(
        project_url, time_series_partition_spec.datetime_partition_column
    )
    multi_prop_json = {
        "datetimePartitionColumn": time_series_partition_spec.datetime_partition_column,
        "detectedMultiseriesIdColumns": [
            {"multiseriesIdColumns": ["series_id"], "timeStep": 1, "timeUnit": "DAY"}
        ],
    }
    responses.add(responses.GET, multi_prop_url, json=multi_prop_json)
    date_feat_json = {
        "projectId": project_id,
        "name": time_series_partition_spec.datetime_partition_column,
        "id": 2,
        "timeSeriesEligible": False,
        "featureType": "date",
        "timeSeriesEligibilityReason": "notUnique",
        "lowInformation": False,
        "uniqueCount": 500,
    }
    responses.add(
        responses.GET,
        "{}features/{}/".format(project_url, time_series_partition_spec.datetime_partition_column),
        json=date_feat_json,
    )

    responses.add(
        responses.PATCH,
        aim_url,
        body="",
        status=202,
        adding_headers={"Location": status_url},
        content_type="application/json",
    )

    responses.add(
        responses.GET,
        status_url,
        body="",
        status=303,
        adding_headers={"Location": project_url},
        content_type="application/json",
    )
    responses.add(
        responses.GET,
        project_url,
        body=project_with_target_json,
        status=200,
        content_type="application/json",
    )

    expected_backtests = [
        {
            "index": bt.index,
            "gapDuration": bt.gap_duration,
            "validationStartDate": bt.validation_start_date.isoformat(),
            "validationDuration": bt.validation_duration,
        }
        for bt in time_series_partition_spec.backtests
    ]
    expected_payload = {
        "datetimePartitionColumn": time_series_partition_spec.datetime_partition_column,
        "multiseriesIdColumns": time_series_partition_spec.multiseries_id_columns,
        "useTimeSeries": time_series_partition_spec.use_time_series,
        "defaultToKnownInAdvance": time_series_partition_spec.default_to_known_in_advance,
        "defaultToDoNotDerive": time_series_partition_spec.default_to_do_not_derive,
        "featureDerivationWindowStart": time_series_partition_spec.feature_derivation_window_start,
        "featureDerivationWindowEnd": time_series_partition_spec.feature_derivation_window_end,
        "forecastWindowStart": time_series_partition_spec.forecast_window_start,
        "forecastWindowEnd": time_series_partition_spec.forecast_window_end,
        "autopilotDataSelectionMethod": time_series_partition_spec.autopilot_data_selection_method,
        "validationDuration": time_series_partition_spec.validation_duration,
        "holdoutStartDate": holdout_start_date.isoformat(),
        "holdoutDuration": time_series_partition_spec.holdout_duration,
        "gapDuration": time_series_partition_spec.gap_duration,
        "numberOfBacktests": time_series_partition_spec.number_of_backtests,
        "backtests": expected_backtests,
        "cvMethod": "datetime",
        "target": "target",
        "mode": "auto",
        "quickrun": True,
        "useCrossSeriesFeatures": True,
        "aggregationType": "total",
        "allowPartialHistoryTimeSeriesPredictions": (
            time_series_partition_spec.allow_partial_history_time_series_predictions
        ),
    }
    project_without_target.set_target("target", partitioning_method=time_series_partition_spec)

    assert responses.calls[2].request.method == "PATCH"
    actual_request = request_body_to_json(responses.calls[2].request)
    assert set(actual_request.keys()) == set(expected_payload.keys())
    assert actual_request == expected_payload


@responses.activate
def test_use_cross_series_group_by_with_columns_validated(
    time_series_partition_spec,
    holdout_start_date,
    project_with_target_json,
    project_without_target,
    unittest_endpoint,
    project_id,
):

    time_series_partition_spec.multiseries_id_columns = ["series_id"]
    time_series_partition_spec.use_cross_series_features = True
    time_series_partition_spec.aggregation_type = "total"
    time_series_partition_spec.cross_series_group_by_columns = ["correct_location"]

    project_url = "{}/projects/{}/".format(unittest_endpoint, project_id)
    aim_url = project_url + "aim/"
    status_url = "{}/status/some-status/".format(unittest_endpoint)
    multi_prop_url = "{}features/{}/multiseriesProperties/".format(
        project_url, time_series_partition_spec.datetime_partition_column
    )
    multi_prop_json = {
        "datetimePartitionColumn": time_series_partition_spec.datetime_partition_column,
        "detectedMultiseriesIdColumns": [
            {"multiseriesIdColumns": ["series_id"], "timeStep": 1, "timeUnit": "DAY"}
        ],
    }
    responses.add(responses.GET, multi_prop_url, json=multi_prop_json)

    responses.add(
        responses.GET,
        "{}features/{}/".format(project_url, time_series_partition_spec.datetime_partition_column),
        json={
            "projectId": project_id,
            "name": time_series_partition_spec.datetime_partition_column,
            "id": 2,
            "timeSeriesEligible": False,
            "featureType": "date",
            "timeSeriesEligibilityReason": "notUnique",
            "lowInformation": False,
            "uniqueCount": 500,
        },
    )

    responses.add(
        responses.PATCH,
        aim_url,
        body="",
        status=202,
        adding_headers={"Location": status_url},
        content_type="application/json",
    )

    responses.add(
        responses.GET,
        status_url,
        body="",
        status=303,
        adding_headers={"Location": project_url},
        content_type="application/json",
    )

    responses.add(
        responses.GET,
        project_url,
        body=project_with_target_json,
        status=200,
        content_type="application/json",
    )

    responses.add(
        responses.GET,
        "{}features/{}/".format(project_url, time_series_partition_spec.multiseries_id_columns[0]),
        json={
            "projectId": project_id,
            "name": time_series_partition_spec.multiseries_id_columns[0],
            "id": 3,
            "timeSeriesEligible": True,
            "featureType": "category",
            "timeSeriesEligibilityReason": "suitable",
            "lowInformation": False,
            "uniqueCount": 100,
        },
    )

    responses.add(
        responses.GET,
        "{}multiseriesIds/{}/crossSeriesProperties/?crossSeriesGroupByColumns={}".format(
            project_url,
            time_series_partition_spec.multiseries_id_columns[0],
            time_series_partition_spec.cross_series_group_by_columns[0],
        ),
        json={
            "multiseriesId": time_series_partition_spec.multiseries_id_columns[0],
            "crossSeriesGroupByColumns": [
                {
                    "name": time_series_partition_spec.cross_series_group_by_columns[0],
                    "eligibility": "suitable",
                    "isEligible": True,
                }
            ],
        },
    )

    expected_backtests = [
        {
            "index": bt.index,
            "gapDuration": bt.gap_duration,
            "validationStartDate": bt.validation_start_date.isoformat(),
            "validationDuration": bt.validation_duration,
        }
        for bt in time_series_partition_spec.backtests
    ]
    expected_payload = {
        "datetimePartitionColumn": time_series_partition_spec.datetime_partition_column,
        "multiseriesIdColumns": time_series_partition_spec.multiseries_id_columns,
        "useTimeSeries": time_series_partition_spec.use_time_series,
        "defaultToKnownInAdvance": time_series_partition_spec.default_to_known_in_advance,
        "defaultToDoNotDerive": time_series_partition_spec.default_to_do_not_derive,
        "featureDerivationWindowStart": time_series_partition_spec.feature_derivation_window_start,
        "featureDerivationWindowEnd": time_series_partition_spec.feature_derivation_window_end,
        "forecastWindowStart": time_series_partition_spec.forecast_window_start,
        "forecastWindowEnd": time_series_partition_spec.forecast_window_end,
        "autopilotDataSelectionMethod": time_series_partition_spec.autopilot_data_selection_method,
        "validationDuration": time_series_partition_spec.validation_duration,
        "holdoutStartDate": holdout_start_date.isoformat(),
        "holdoutDuration": time_series_partition_spec.holdout_duration,
        "gapDuration": time_series_partition_spec.gap_duration,
        "numberOfBacktests": time_series_partition_spec.number_of_backtests,
        "backtests": expected_backtests,
        "cvMethod": "datetime",
        "target": "target",
        "mode": "auto",
        "quickrun": True,
        "useCrossSeriesFeatures": True,
        "aggregationType": "total",
        "crossSeriesGroupByColumns": time_series_partition_spec.cross_series_group_by_columns,
        "allowPartialHistoryTimeSeriesPredictions": (
            time_series_partition_spec.allow_partial_history_time_series_predictions
        ),
    }
    project_without_target.set_target("target", partitioning_method=time_series_partition_spec)

    assert responses.calls[4].request.method == "PATCH"
    actual_request = request_body_to_json(responses.calls[4].request)
    assert set(actual_request.keys()) == set(expected_payload.keys())
    assert actual_request == expected_payload


@responses.activate
def test_use_cross_series_group_by_with_columns_not_validated(
    time_series_partition_spec,
    project_with_target_json,
    project_without_target,
    holdout_start_date,
    unittest_endpoint,
    project_id,
):
    time_series_partition_spec.multiseries_id_columns = ["series_id"]
    time_series_partition_spec.use_cross_series_features = True
    time_series_partition_spec.aggregation_type = "total"
    time_series_partition_spec.cross_series_group_by_columns = ["location"]

    project_url = "{}/projects/{}/".format(unittest_endpoint, project_id)
    aim_url = project_url + "aim/"
    status_url = "{}/status/some-status/".format(unittest_endpoint)
    multi_prop_url = "{}features/{}/multiseriesProperties/".format(
        project_url, time_series_partition_spec.datetime_partition_column
    )
    cross_series_url = "{}/projects/{}/crossSeriesProperties/".format(unittest_endpoint, project_id)

    multi_prop_json = {
        "datetimePartitionColumn": time_series_partition_spec.datetime_partition_column,
        "detectedMultiseriesIdColumns": [
            {"multiseriesIdColumns": ["series_id"], "timeStep": 1, "timeUnit": "DAY"}
        ],
    }

    responses.add(responses.GET, multi_prop_url, json=multi_prop_json)

    responses.add(
        responses.GET,
        "{}features/{}/".format(project_url, time_series_partition_spec.datetime_partition_column),
        json={
            "projectId": project_id,
            "name": time_series_partition_spec.datetime_partition_column,
            "id": 2,
            "timeSeriesEligible": False,
            "featureType": "date",
            "timeSeriesEligibilityReason": "notUnique",
            "lowInformation": False,
            "uniqueCount": 500,
        },
    )

    responses.add(
        responses.PATCH,
        aim_url,
        body="",
        status=202,
        adding_headers={"Location": status_url},
        content_type="application/json",
    )

    responses.add(
        responses.GET,
        status_url,
        body="",
        status=303,
        adding_headers={"Location": project_url},
        content_type="application/json",
    )

    responses.add(
        responses.GET,
        project_url,
        body=project_with_target_json,
        status=200,
        content_type="application/json",
    )

    responses.add(
        responses.GET,
        "{}features/{}/".format(project_url, time_series_partition_spec.multiseries_id_columns[0]),
        json={
            "projectId": project_id,
            "name": time_series_partition_spec.multiseries_id_columns[0],
            "id": 3,
            "timeSeriesEligible": True,
            "featureType": "category",
            "timeSeriesEligibilityReason": "suitable",
            "lowInformation": False,
            "uniqueCount": 100,
        },
    )

    responses.add(
        responses.GET,
        "{}multiseriesIds/{}/crossSeriesProperties/?crossSeriesGroupByColumns={}".format(
            project_url,
            time_series_partition_spec.multiseries_id_columns[0],
            time_series_partition_spec.cross_series_group_by_columns[0],
        ),
        json={
            "multiseriesId": time_series_partition_spec.multiseries_id_columns[0],
            "crossSeriesGroupByColumns": [
                {
                    "name": time_series_partition_spec.cross_series_group_by_columns[0],
                    "eligibility": "notAnalyzed",
                    "isEligible": False,
                }
            ],
        },
    )

    responses.add(
        responses.POST,
        cross_series_url,
        body="",
        status=202,
        adding_headers={"Location": status_url},
        content_type="application/json",
    )

    responses.add(
        responses.GET,
        status_url,
        body="",
        status=303,
        adding_headers={"Location": project_url},
        content_type="application/json",
    )

    responses.add(
        responses.GET,
        "{}multiseriesIds/{}/crossSeriesProperties/?crossSeriesGroupByColumns={}".format(
            project_url,
            time_series_partition_spec.multiseries_id_columns[0],
            time_series_partition_spec.cross_series_group_by_columns[0],
        ),
        json={
            "multiseriesId": time_series_partition_spec.multiseries_id_columns[0],
            "crossSeriesGroupByColumns": [
                {
                    "name": time_series_partition_spec.cross_series_group_by_columns[0],
                    "eligibility": "suitable",
                    "isEligible": True,
                }
            ],
        },
    )

    expected_backtests = [
        {
            "index": bt.index,
            "gapDuration": bt.gap_duration,
            "validationStartDate": bt.validation_start_date.isoformat(),
            "validationDuration": bt.validation_duration,
        }
        for bt in time_series_partition_spec.backtests
    ]
    expected_payload = {
        "datetimePartitionColumn": time_series_partition_spec.datetime_partition_column,
        "multiseriesIdColumns": time_series_partition_spec.multiseries_id_columns,
        "useTimeSeries": time_series_partition_spec.use_time_series,
        "defaultToKnownInAdvance": time_series_partition_spec.default_to_known_in_advance,
        "defaultToDoNotDerive": time_series_partition_spec.default_to_do_not_derive,
        "featureDerivationWindowStart": time_series_partition_spec.feature_derivation_window_start,
        "featureDerivationWindowEnd": time_series_partition_spec.feature_derivation_window_end,
        "forecastWindowStart": time_series_partition_spec.forecast_window_start,
        "forecastWindowEnd": time_series_partition_spec.forecast_window_end,
        "autopilotDataSelectionMethod": time_series_partition_spec.autopilot_data_selection_method,
        "validationDuration": time_series_partition_spec.validation_duration,
        "holdoutStartDate": holdout_start_date.isoformat(),
        "holdoutDuration": time_series_partition_spec.holdout_duration,
        "gapDuration": time_series_partition_spec.gap_duration,
        "numberOfBacktests": time_series_partition_spec.number_of_backtests,
        "backtests": expected_backtests,
        "cvMethod": "datetime",
        "target": "target",
        "mode": "auto",
        "quickrun": True,
        "useCrossSeriesFeatures": True,
        "aggregationType": "total",
        "crossSeriesGroupByColumns": time_series_partition_spec.cross_series_group_by_columns,
        "allowPartialHistoryTimeSeriesPredictions": (
            time_series_partition_spec.allow_partial_history_time_series_predictions
        ),
    }

    project_without_target.set_target("target", partitioning_method=time_series_partition_spec)

    assert responses.calls[7].request.method == "PATCH"
    actual_request = request_body_to_json(responses.calls[7].request)
    assert set(actual_request.keys()) == set(expected_payload.keys())
    assert actual_request == expected_payload


@responses.activate
def test_use_cross_series_group_by_with_invalid_columns(
    time_series_partition_spec,
    project_with_target_json,
    project_without_target,
    unittest_endpoint,
    project_id,
):

    time_series_partition_spec.multiseries_id_columns = ["series_id"]
    time_series_partition_spec.use_cross_series_features = True
    time_series_partition_spec.aggregation_type = "total"
    time_series_partition_spec.cross_series_group_by_columns = ["location"]

    project_url = "{}/projects/{}/".format(unittest_endpoint, project_id)
    aim_url = project_url + "aim/"
    status_url = "{}/status/some-status/".format(unittest_endpoint)
    multi_prop_url = "{}features/{}/multiseriesProperties/".format(
        project_url, time_series_partition_spec.datetime_partition_column
    )
    multi_prop_json = {
        "datetimePartitionColumn": time_series_partition_spec.datetime_partition_column,
        "detectedMultiseriesIdColumns": [
            {"multiseriesIdColumns": ["series_id"], "timeStep": 1, "timeUnit": "DAY"}
        ],
    }
    responses.add(responses.GET, multi_prop_url, json=multi_prop_json)

    responses.add(
        responses.GET,
        "{}features/{}/".format(project_url, time_series_partition_spec.datetime_partition_column),
        json={
            "projectId": project_id,
            "name": time_series_partition_spec.datetime_partition_column,
            "id": 2,
            "timeSeriesEligible": False,
            "featureType": "date",
            "timeSeriesEligibilityReason": "notUnique",
            "lowInformation": False,
            "uniqueCount": 500,
        },
    )

    responses.add(
        responses.PATCH,
        aim_url,
        body="",
        status=202,
        adding_headers={"Location": status_url},
        content_type="application/json",
    )

    responses.add(
        responses.GET,
        status_url,
        body="",
        status=303,
        adding_headers={"Location": project_url},
        content_type="application/json",
    )

    responses.add(
        responses.GET,
        project_url,
        body=project_with_target_json,
        status=200,
        content_type="application/json",
    )

    responses.add(
        responses.GET,
        "{}features/{}/".format(project_url, time_series_partition_spec.multiseries_id_columns[0]),
        json={
            "projectId": project_id,
            "name": time_series_partition_spec.multiseries_id_columns[0],
            "id": 3,
            "timeSeriesEligible": True,
            "featureType": "category",
            "timeSeriesEligibilityReason": "suitable",
            "lowInformation": False,
            "uniqueCount": 100,
        },
    )

    responses.add(
        responses.GET,
        "{}multiseriesIds/{}/crossSeriesProperties/?crossSeriesGroupByColumns={}".format(
            project_url,
            time_series_partition_spec.multiseries_id_columns[0],
            time_series_partition_spec.cross_series_group_by_columns[0],
        ),
        json={
            "multiseriesId": time_series_partition_spec.multiseries_id_columns[0],
            "crossSeriesGroupByColumns": [
                {
                    "name": time_series_partition_spec.cross_series_group_by_columns[0],
                    "eligibility": "splitSeries",
                    "isEligible": False,
                }
            ],
        },
    )

    try:
        project_without_target.set_target("target", partitioning_method=time_series_partition_spec)
        assert False, "Should fail with exception as column '{}' is not eligible.".format(
            time_series_partition_spec.cross_series_group_by_columns[0]
        )
    except InvalidUsageError:
        assert True



import json

import pytest
import responses

from datarobot import Blueprint
from datarobot.errors import ClientError
from datarobot.models import BlueprintChart, BlueprintTaskDocument, ModelBlueprintChart
from datarobot.utils import from_api


@pytest.fixture
def blueprint_id():
    return "e1c7fc29ba2e612a72272324b8a842af"


@pytest.fixture
def featurelist_id():
    return "e1c7fc29ba2e612a72272324b8a923ba"


@pytest.fixture
def blueprint_data(blueprint_id, project_id):
    return {
        "modelType": "RandomForest Regressor",
        "processes": ["Missing values imputed"],
        "projectId": project_id,
        "id": blueprint_id,
        "blueprintCategory": "DataRobot",
    }


@pytest.fixture
def blueprint_url(project_id, blueprint_id):
    return "https://host_name.com/projects/{}/blueprints/{}/".format(project_id, blueprint_id)


@pytest.fixture
def blueprint_chart_dot():
    return """digraph "Blueprint Chart" {
graph [rankdir=LR]
1 [label="Task1"]
2 [label="Task2"]
1 -> 2
}"""


@pytest.fixture
def blueprint_docs_url(project_id, blueprint_id):
    return "https://host_name.com/projects/{}/blueprints/{}/blueprintDocs/".format(
        project_id, blueprint_id
    )


@pytest.fixture()
def broken_blueprint_docs(blueprint_docs):
    bad_data = {"some_broken_field": "some_broken_value"}
    bad_doc = blueprint_docs[0].copy()
    bad_doc.update(bad_data)
    bad_doc["parameters"][0].update(bad_data)
    bad_doc["references"][0].update(bad_data)
    bad_doc["links"][0].update(bad_data)
    return [bad_doc]


@pytest.fixture
def blueprint_chart_url(project_id, blueprint_id):
    return "https://host_name.com/projects/{}/blueprints/{}/blueprintChart/".format(
        project_id, blueprint_id
    )


def test_instantiation(blueprint_data):
    bp = Blueprint.from_data(from_api(blueprint_data))

    assert bp.model_type == blueprint_data["modelType"]
    assert bp.processes == blueprint_data["processes"]
    assert bp.project_id == blueprint_data["projectId"]
    assert bp.id == blueprint_data["id"]
    assert bp.blueprint_category == blueprint_data["blueprintCategory"]


def test_future_proof(blueprint_data):
    Blueprint.from_data(dict(from_api(blueprint_data), new_key="future"))


def test_non_ascii(blueprint_data, unicode_string):
    bp = Blueprint.from_server_data(dict(blueprint_data, modelType=unicode_string))
    print(bp)  # test that __repr__ works correctly - this used to fail


@responses.activate
def test_blueprint_get(blueprint_data, project_id, blueprint_url, blueprint_id):
    responses.add(
        responses.GET,
        blueprint_url,
        status=200,
        content_type="application/json",
        body=json.dumps(blueprint_data),
    )
    bp = Blueprint.get(project_id, blueprint_id)

    assert bp.model_type == blueprint_data["modelType"]
    assert bp.processes == blueprint_data["processes"]
    assert bp.project_id == blueprint_data["projectId"]
    assert bp.id == blueprint_data["id"]
    assert bp.recommended_featurelist_id is None


@responses.activate
def test_blueprint_get_with_recommended_featurelist_id(
    blueprint_data, project_id, blueprint_url, blueprint_id, featurelist_id
):
    blueprint_data["recommended_featurelist_id"] = featurelist_id
    responses.add(
        responses.GET,
        blueprint_url,
        status=200,
        content_type="application/json",
        body=json.dumps(blueprint_data),
    )
    bp = Blueprint.get(project_id, blueprint_id)
    assert bp.id == blueprint_data["id"]
    assert bp.recommended_featurelist_id == featurelist_id


@responses.activate
def test_blueprint_get_not_found(project_id, blueprint_url, blueprint_id):
    responses.add(
        responses.GET,
        blueprint_url,
        status=404,
        content_type="application/json",
        body=json.dumps({"message": "No blueprint data found"}),
    )
    with pytest.raises(ClientError):
        Blueprint.get(project_id, blueprint_id)


@responses.activate
def test_blueprint_chart_get(blueprint_chart_data, project_id, blueprint_chart_url, blueprint_id):
    responses.add(
        responses.GET,
        blueprint_chart_url,
        status=200,
        content_type="application/json",
        body=json.dumps(blueprint_chart_data),
    )
    bp = BlueprintChart.get(project_id, blueprint_id)

    assert bp.nodes == blueprint_chart_data["nodes"]
    assert bp.edges == blueprint_chart_data["edges"]


def test_blueprint_chart_to_graphviz(blueprint_chart_data, blueprint_chart_dot):
    bp = BlueprintChart(**blueprint_chart_data)
    assert bp.to_graphviz() == blueprint_chart_dot


@responses.activate
def test_blueprint_get_documents(blueprint_docs_url, blueprint_docs, project_id, blueprint_id):
    responses.add(
        responses.GET,
        blueprint_docs_url,
        status=200,
        content_type="application/json",
        body=json.dumps(blueprint_docs),
    )
    bp = Blueprint.from_data({"id": blueprint_id, "project_id": project_id})
    docs = bp.get_documents()
    assert len(docs) == 1
    assert isinstance(docs[0], BlueprintTaskDocument)
    assert docs[0].title == blueprint_docs[0]["title"]
    assert docs[0].task == blueprint_docs[0]["task"]
    assert docs[0].description == blueprint_docs[0]["description"]
    assert docs[0].links == blueprint_docs[0]["links"]
    assert docs[0].parameters == blueprint_docs[0]["parameters"]
    assert docs[0].references == blueprint_docs[0]["references"]


@responses.activate
def test_blueprint_get_documents_future_proof(
    blueprint_docs_url, broken_blueprint_docs, project_id, blueprint_id
):
    responses.add(
        responses.GET,
        blueprint_docs_url,
        status=200,
        content_type="application/json",
        body=json.dumps(broken_blueprint_docs),
    )
    bp = Blueprint.from_data({"id": blueprint_id, "project_id": project_id})
    docs = bp.get_documents()
    assert len(docs) == 1
    assert isinstance(docs[0], BlueprintTaskDocument)


@responses.activate
def test_model_blueprint_chart_get(blueprint_chart_data, project_id, model_id):
    responses.add(
        responses.GET,
        "https://host_name.com/projects/{}/models/{}/blueprintChart/".format(project_id, model_id),
        status=200,
        content_type="application/json",
        body=json.dumps(blueprint_chart_data),
    )
    model_bp = ModelBlueprintChart.get(project_id, model_id)

    assert model_bp.nodes == blueprint_chart_data["nodes"]
    assert model_bp.edges == blueprint_chart_data["edges"]



import json

import responses
from six.moves.urllib.parse import urljoin

from datarobot.models.visualai.augmentation import (
    ImageAugmentationList,
    ImageAugmentationOptions,
    ImageAugmentationSample,
)


@responses.activate
def test_augmentation_options(visualai_aug_options, augmentation_options_url, aug_pid):
    responses.add(
        responses.GET,
        augmentation_options_url,
        status=200,
        content_type="application/json",
        body=json.dumps(visualai_aug_options),
    )
    options = ImageAugmentationOptions.get(aug_pid)
    assert isinstance(options, ImageAugmentationOptions)

    from datarobot.utils import from_api

    data = from_api(visualai_aug_options)
    expected = ImageAugmentationOptions(**data)
    assert options.project_id == expected.project_id
    assert options.id == expected.id


@responses.activate
def test_augmentation_list_get(visualai_augmentation_list, image_augmentations_url):
    aug_id = visualai_augmentation_list["id"]
    aug_list_url = urljoin(image_augmentations_url, str(aug_id) + "/")
    responses.add(
        responses.GET,
        aug_list_url,
        status=200,
        content_type="application/json",
        body=json.dumps(visualai_augmentation_list),
    )
    aug_list = ImageAugmentationList.get(aug_id)
    assert isinstance(aug_list, ImageAugmentationList)
    assert aug_list.id == visualai_augmentation_list["id"]
    assert aug_list.project_id == visualai_augmentation_list["projectId"]


@responses.activate
def test_augmentation_list_create(visualai_augmentation_list, image_augmentations_url):
    aug_id = visualai_augmentation_list["id"]
    responses.add(
        responses.POST,
        image_augmentations_url,
        status=200,
        content_type="application/json",
        body=json.dumps({"augmentationListId": aug_id}),
    )
    aug_list_url = urljoin(image_augmentations_url, str(aug_id) + "/")
    responses.add(
        responses.GET,
        aug_list_url,
        status=200,
        content_type="application/json",
        body=json.dumps(visualai_augmentation_list),
    )
    from datarobot.utils import from_api

    data = from_api(visualai_augmentation_list)
    del data["id"]
    name = data.pop("name")
    pid = data.pop("project_id")
    new_aug_list = ImageAugmentationList.create(name, pid, **data)
    assert isinstance(new_aug_list, ImageAugmentationList)


@responses.activate
def test_augmentation_sample_compute(
    visualai_augmentation_list, augmentation_samples_url, status_url, status_id
):
    responses.add(
        responses.POST,
        augmentation_samples_url,
        content_type="application/json",
        body=json.dumps({"status_id": status_id}),
        status=202,
        adding_headers={"Location": status_url},
    )
    from datarobot.utils import from_api

    data = from_api(visualai_augmentation_list)
    control_list = ImageAugmentationList(**data)
    ret_status_url = ImageAugmentationSample.compute(control_list, 4)
    assert ret_status_url == status_url


@responses.activate
def test_augmentation_sample_retrieve(augmentation_samples_url):
    sample_url = urljoin(augmentation_samples_url, "5e7e562528513130ab237875/")
    a_sample = {
        "imageId": "1234",
        "width": 256,
        "height": 256,
        "originalImageId": "5678",
        "project_id": "1234",
    }
    ret_data = {"totalCount": 1, "next": None, "previous": None, "data": [a_sample]}
    responses.add(responses.GET, sample_url, body=json.dumps(ret_data), status=200)

    ret_sample = ImageAugmentationSample.list("5e7e562528513130ab237875")
    assert len(ret_sample) == 1
    assert ret_sample[0].image_id == a_sample["imageId"]
    assert ret_sample[0].height == a_sample["height"]
    assert ret_sample[0].width == a_sample["width"]
    assert ret_sample[0].original_image_id == a_sample["originalImageId"]



def assert_version(version, version_json):
    assert version.id == version_json["id"]
    assert version.environment_id == version_json["environmentId"]
    assert version.label == version_json["label"]
    assert version.description == version_json["description"]
    assert version.build_status == version_json["buildStatus"]
    assert version.created_at == version_json["created"]
    assert version.docker_context_size == version_json.get("dockerContextSize")
    assert version.docker_image_size == version_json.get("dockerImageSize")


def assert_custom_model_version(version, version_json):
    assert version.id == version_json["id"]
    assert version.custom_model_id == version_json["customModelId"]
    assert version.label == version_json["label"]
    assert version.description == version_json["description"]
    assert version.version_minor == version_json["versionMinor"]
    assert version.version_major == version_json["versionMajor"]
    assert version.is_frozen == version_json["isFrozen"]
    assert version.base_environment_id == version_json.get("baseEnvironmentId")
    assert version.base_environment_version_id == version_json.get("baseEnvironmentVersionId")
    if version.base_environment_id is not None:
        assert version.base_environment_version_id is not None

    assert len(version.items) == len(version_json["items"])
    for item, item_json in zip(version.items, version_json["items"]):
        assert item.id == item_json["id"]
        assert item.file_name == item_json["fileName"]
        assert item.file_path == item_json["filePath"]
        assert item.file_source == item_json["fileSource"]
        assert item.created_at == item_json["created"]

    assert len(version.dependencies) == len(version_json.get("dependencies", []))
    for dependency, dependency_json in zip(
        version.dependencies, version_json.get("dependencies", [])
    ):
        assert dependency.package_name == dependency_json["packageName"]
        assert dependency.line == dependency_json["line"]
        assert dependency.line_number == dependency_json["lineNumber"]

        for constraint, constraint_json in zip(
            dependency.constraints, dependency_json["constraints"]
        ):
            assert constraint.version == constraint_json["version"]
            assert constraint.constraint_type == constraint_json["constraintType"]

    assert version.created_at == version_json["created"]
    assert version.network_egress_policy == version_json["networkEgressPolicy"]
    assert version.maximum_memory == version_json["maximumMemory"]
    assert version.replicas == version_json["replicas"]
    assert version.required_metadata == version_json.get("requiredMetadata")


def assert_custom_model_version_dependency_build(
    build_info, build_info_json, custom_model_id, custom_model_version_id
):
    assert build_info.custom_model_id == custom_model_id
    assert build_info.custom_model_version_id == custom_model_version_id

    assert build_info.started_at == build_info_json["buildStart"]
    assert build_info.completed_at == build_info_json["buildEnd"]
    assert build_info.build_status == build_info_json["buildStatus"]



import json

import pytest
import responses

from datarobot import ModelRecommendation
from datarobot.enums import RECOMMENDED_MODEL_TYPE


@pytest.fixture
def list_recommended_model_json():
    model_list = [
        {
            "recommendationType": RECOMMENDED_MODEL_TYPE.MOST_ACCURATE,
            "modelId": "model1",
            "projectId": "project1",
        },
        {
            "recommendationType": RECOMMENDED_MODEL_TYPE.FAST_ACCURATE,
            "modelId": "model2",
            "projectId": "project1",
        },
    ]
    return json.dumps(model_list)


@pytest.fixture
def recommended_model_json():
    model = {
        "recommendationType": "Fast & Accurate",
        "modelId": "5ad8e121fcf4c87d96ae02dd",
        "projectId": "project1",
    }
    return json.dumps(model)


@pytest.fixture
def recommended_model_extra_field_json():
    model = {
        "recommendationType": "Fast & Accurate",
        "modelId": "5ad8e121fcf4c87d96ae02dd",
        "projectId": "project1",
        "extraField": "value",
    }
    return json.dumps(model)


@responses.activate
def test_recommended_model_default(recommended_model_json):
    responses.add(
        responses.GET,
        "https://host_name.com/projects/p-id/recommendedModels/recommendedModel/",
        body=recommended_model_json,
    )
    recommended_model = ModelRecommendation.get("p-id")
    assert recommended_model.recommendation_type == "Fast & Accurate"
    assert recommended_model.model_id == "5ad8e121fcf4c87d96ae02dd"
    assert recommended_model.project_id == "project1"


@responses.activate
def test_recommended_model_specific(list_recommended_model_json):
    responses.add(
        responses.GET,
        "https://host_name.com/projects/p-id/recommendedModels/",
        body=list_recommended_model_json,
    )
    recommended_model = ModelRecommendation.get("p-id", RECOMMENDED_MODEL_TYPE.MOST_ACCURATE)
    assert recommended_model.recommendation_type == "Most Accurate"
    assert recommended_model.model_id == "model1"
    assert recommended_model.project_id == "project1"


@responses.activate
def test_recommended_model_ignore_extra_fields(recommended_model_extra_field_json):
    responses.add(
        responses.GET,
        "https://host_name.com/projects/p-id/recommendedModels/recommendedModel/",
        body=recommended_model_extra_field_json,
    )
    recommended_model = ModelRecommendation.get("p-id")
    assert recommended_model.recommendation_type == "Fast & Accurate"
    assert recommended_model.model_id == "5ad8e121fcf4c87d96ae02dd"
    assert recommended_model.project_id == "project1"
    assert not hasattr(recommended_model, "extraField")


@responses.activate
def test_recommended_model_list(list_recommended_model_json):
    responses.add(
        responses.GET,
        "https://host_name.com/projects/p-id/recommendedModels/",
        body=list_recommended_model_json,
    )
    recommended_models = ModelRecommendation.get_all("p-id")
    assert len(recommended_models) == 2
    expected_models = json.loads(list_recommended_model_json)

    expected_model1 = next(model for model in expected_models if model["modelId"] == "model1")
    expected_model2 = next(model for model in expected_models if model["modelId"] == "model2")
    actual_model1 = [model for model in recommended_models if model.model_id == "model1"]
    actual_model2 = [model for model in recommended_models if model.model_id == "model2"]

    assert len(actual_model1) == 1
    assert len(actual_model2) == 1
    actual_model1 = actual_model1[0]
    actual_model2 = actual_model2[0]

    assert actual_model1.recommendation_type == expected_model1["recommendationType"]
    assert actual_model2.recommendation_type == expected_model2["recommendationType"]
    assert actual_model1.project_id == expected_model1["projectId"]
    assert actual_model2.project_id == expected_model2["projectId"]

    assert (
        ModelRecommendation.get_recommendation(
            recommended_models, RECOMMENDED_MODEL_TYPE.MOST_ACCURATE
        )
        is not None
    )

    assert (
        ModelRecommendation.get_recommendation(
            recommended_models, RECOMMENDED_MODEL_TYPE.RECOMMENDED_FOR_DEPLOYMENT
        )
        is None
    )
